@@ -7,47 +7,62 @@ from aioviberbot.api.consts import BOT_API_ENDPOINT
 
 
 class ApiRequestSender:
-    def __init__(self, logger, viber_bot_api_url, bot_configuration, viber_bot_user_agent):
+    def __init__(
+            self,
+            logger,
+            viber_bot_api_url,
+            bot_configuration,
+            viber_bot_user_agent,
+            client_session=None,
+    ):
         self._logger = logger
         self._viber_bot_api_url = viber_bot_api_url
         self._bot_configuration = bot_configuration
         self._user_agent = viber_bot_user_agent
+        self._client_session = client_session
 
     async def post_request(self, endpoint, payload=None):
         url = self._viber_bot_api_url + '/' + endpoint
         payload = payload or {}
         payload['auth_token'] = self._bot_configuration.auth_token
         headers = {'User-Agent': self._user_agent}
-        async with aiohttp.ClientSession() as session:
-            try:
-                response = await session.post(
-                    url=url,
-                    json=payload,
-                    headers=headers,
-                    timeout=10,
-                )
-                response.raise_for_status()
-                result = await response.json()
-            except (aiohttp.ClientError, asyncio.TimeoutError):
-                self._logger.error(
-                    'failed to post request to endpoint={0}, with payload={1}. error is: {2}'
-                    .format(endpoint, payload, traceback.format_exc()),
-                )
-                raise
-            except Exception:
-                self._logger.error(
-                    'unexpected Exception while trying to post request. error is: {0}'
-                    .format(traceback.format_exc()),
-                )
-                raise
-            else:
-                if result['status'] == 0:
-                    return result
 
-                raise Exception(
-                    'failed with status: {0}, message: {1}'
-                    .format(result['status'], result.get('status_message')),
-                )
+        if self._client_session:
+            session = self._client_session
+        else:
+            session = aiohttp.ClientSession(timeout=aiohttp.ClientTimeout(10))
+
+        try:
+            response = await session.post(
+                url=url,
+                json=payload,
+                headers=headers,
+            )
+            response.raise_for_status()
+            result = await response.json()
+        except (aiohttp.ClientError, asyncio.TimeoutError):
+            self._logger.error(
+                'failed to post request to endpoint={0}, with payload={1}. error is: {2}'
+                .format(endpoint, payload, traceback.format_exc()),
+            )
+            raise
+        except Exception:
+            self._logger.error(
+                'unexpected Exception while trying to post request. error is: {0}'
+                .format(traceback.format_exc()),
+            )
+            raise
+        else:
+            if result['status'] == 0:
+                return result
+
+            raise Exception(
+                'failed with status: {0}, message: {1}'
+                .format(result['status'], result.get('status_message')),
+            )
+        finally:
+            if not self._client_session:
+                await session.close()
 
     async def set_webhook(self, url, webhook_events=None, is_inline=False):
         payload = {
